#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé le Tue Sep 20 08:39:17 2022

@auteur: David ALBERTO
(www.astrolabe-science.fr)

Ce script trace un diagramme solaire en projection stéréographique,
pour la latitude définie par l'utilisateur.
Il peut être imprimé et servir de cadran solaire, en plaçant
une tige verticale (le style) au centre du tracé. Le diagramme doit être placé horizontalement,
et orienté selon les points cardinaux.
L'intersection de l'ombre du style avec l'arc diurne de la date indique l'heure solaire.
"""
#  Appel des modules Python :
import matplotlib.pyplot as plt
import numpy as np
from math import sin, cos, tan, acos, atan, pi
import matplotlib.image as image
import matplotlib.pyplot as plt
from matplotlib.offsetbox import OffsetImage, AnnotationBbox

# paramètres à personnaliser ---------------------
plt.rcParams["font.family"] = "Roboto"
phi = 49.5 # latitude en degrés
R = 10 # rayon du diagrammme
coul_arcs_diurnes = 'royalblue'
coul_heures = 'orangered'
prop_arcs_diurnes=dict(color=coul_arcs_diurnes,lw=0.5) # couleur et épaisseur de ligne
prop_heures=dict(color=coul_heures,lw=0.5)  # couleur et épaisseur de ligne
y_offset_style = -0.25*R # décalage du segment de style par rapport au centre

# ------------------------------------
eps=23.44 # obliquité de l'écliptique
eps_rad = np.radians(eps)
phi_str=str(phi) # pour le nom de fichier
NomFichier = "DiagrammeSolaireStereo" + phi_str
phi=np.radians(phi)

#  Calcul de l'angle horaire maximal (degrés):
if phi < pi/2-eps_rad:# si latitude non polaire
    maxH=np.degrees(acos(-tan(phi)*tan(eps_rad)))#angle horaire maxi au 21 juin, pour les heures de lever/coucher
    maxH=int(maxH/15)*15
else:maxH=180

yS = -R*tan((pi/2-phi)/2) # ordonnée du pôle Sud
if phi!=0:
    yM = yS + R/sin(phi) # ordonnée du point M, pour les cercles horaires
else:
    yM=0
#  -------------------------------------------
# Fonctions :
def heure(H):
    """
    à partir de l'angle horaire H, en radians, renvoie l'heure (string)
    """
    H = np.degrees(H)
    heure =str(round(12 + H/15))
    return heure

def coordXY(decl,H):
    """
    à partir de la déclinaison (en rad) et de l'angle horaire (rad),
    renvoie les coordonnées x, y (tuple) d'un point sur le diagramme.
    """
    X = R*sin(H)/(1/cos(decl) + sin(phi) * tan(decl) + cos(phi) * cos(H))
    Y = R * (sin(phi) * cos(H) - cos(phi) * tan(decl)) / (1/cos(decl) + sin(phi) * tan(decl) + cos(phi) * cos(H))
    return X, Y

# --------------------------------------

"""
Paramètres pour maîtriser la taille du tracé en échelle 1 :
Sur le document final, le cercle est à l'échelle 1, cad que son rayon en cm
est exactement égal à la valeur de R définie en début de script.
Les marges gauche et basse sont définies ci-dessous, les autres marges 
sont déduites des premières et des dimensions.

"""

# limites des graduations :
xmin, xmax = -R, R # donc largeur 2*R cm
ymin, ymax = -R, R # donc hauteur 2*R cm

inch = 2.54 # conversion des cm en pouces
fig_width = 21 / inch # largeur de la figure
fig_height = 21 / inch # hauteur de la figure
marge_G = 0.5 / inch # marge gauche (cm convertis en pouces)
largeur_axe = 2*R / inch # en cohérence avec xmin et xmax
hauteur_axe = 2*R / inch # en cohérence avec ymin et ymax
marge_D=fig_width-marge_G-largeur_axe
marge_B= 0.5 / inch # marge du bas, en cm convertis en pouces
marge_H=fig_height-marge_B-hauteur_axe

# --------------------------------------
#  paramètres du graphique :
fig = plt.figure(figsize=(fig_width, fig_height))

# ajustement des dimensions de l'axe :
# les valeurs "left", "right", "bottom" et "top" sont des % entre 0 et 1
plt.subplots_adjust(
    left=marge_G / fig_width,
    right=1-marge_D/fig_width,
    bottom= marge_B / fig_height,
    top= 1 - marge_H / fig_height,
)

ax=plt.subplot(111)
ax.axis('off')# suppression du cadre (axes)
ax.set_aspect('equal')# échelles identiques
ax.set_xlim(-R,R)
ax.set_ylim(-R,R)
ax.set_xticks([])
ax.set_yticks([])

#  --------------------------------------------------

# axes
ax.plot([-R,R],[0,0],c='gray',lw=0.5)
ax.plot([0,0],[-R,R],c='gray',lw=0.5)

horizon = plt.Circle((0,0), radius=R,fill=False,color='g')
ax.add_patch(horizon)

# points cardinaux :
ax.text(0,R,'N',va='top',ha='center')
ax.text(0,-R,'S',va='bottom',ha='center')
ax.text(-R,0,'O',va='center',ha='left')
ax.text(R,0,'E',va='center',ha='right')

# latitude :
ax.text(0,-0.93*R,phi_str.replace('.', ',') + '°',c='red',ha='center',fontsize=15)

# -------------------centre

ax.scatter(0,0,s=20,marker='s',c='r',zorder=3)

# arcs diurnes :-------------------------------------------

for decl in np.radians([-20.15,-11.47,0,11.47,20.15]):
    if sin(phi)+sin(decl)!=0:
        yC = -R*cos(phi)/(sin(phi)+sin(decl))
        rayon = R*cos(decl)/(sin(phi)+sin(decl))
        if decl==0:
            prop=dict(color=coul_arcs_diurnes,lw=1.0)
        else :
            prop=prop_arcs_diurnes
        arc_decl = plt.Circle((0,yC), radius=rayon,fill=False,**prop)
        arc = ax.add_patch(arc_decl)
        arc.set_clip_path(horizon)

# arc été :
decl = np.radians(eps)
yC = -R*cos(phi)/(sin(phi)+sin(decl))
rayon = R*cos(decl)/(sin(phi)+sin(decl))
arc_ete =ax.add_patch(plt.Circle((0,yC), radius=rayon,fill=False,**prop_arcs_diurnes))
arc_ete.set_clip_path(horizon)
# arc hiver :
decl = np.radians(-eps)
yC = -R*cos(phi)/(sin(phi)+sin(decl))
rayon = R*cos(decl)/(sin(phi)+sin(decl))
arc_hiver =ax.add_patch(plt.Circle((0,yC), radius=rayon,fill=False,**prop_arcs_diurnes))
arc_hiver.set_clip_path(horizon)

# --------------- fin des arcs diurnes

#  lignes horaires :

for H in np.radians(range(-maxH,maxH+15,15)):
    liste_x=[]
    liste_y=[]
    if phi!=0:
        decl_min = atan(-cos(H) / tan(phi))# déclinaison minimale pour un Soleil levé
    else: decl_min=-eps_rad
    for decl in np.linspace(-eps_rad, eps_rad,100):
        if decl >= decl_min:
            X,Y = coordXY(decl, H)
            liste_x.append(X)
            liste_y.append(Y)
    ax.plot(liste_x,liste_y,**prop_heures)
    if H == 0:
        ax.text(min(liste_x),min(liste_y)-0.2,heure(H),ha='center',va='top',c=coul_heures)
    elif H > 0:
        ax.text(min(liste_x)-0.3,min(liste_y)-0.2,heure(H),ha='center',va='top',c=coul_heures)
    else:ax.text(max(liste_x)+0.3,min(liste_y)-0.2,heure(H),ha='center',va='top',c=coul_heures)


# demi-heures :
for H in np.radians(np.arange(-maxH+7.5,maxH+7.5,15)):
    liste_x=[]
    liste_y=[]
    if phi!=0:
        decl_min = atan(-cos(H) / tan(phi))# déclinaison minimale pour un Soleil levé
    else: decl_min=-eps_rad
    for decl in np.linspace(-eps_rad, eps_rad,100):
        if decl >= decl_min:
            X,Y = coordXY(decl, H)
            liste_x.append(X)
            liste_y.append(Y)
    ax.plot(liste_x,liste_y,c=coul_heures,lw=0.5,alpha=0.2)

#  fin des lignes horaires
#  -----------------------------------

#  longueur minimale du style :
l_min = R*tan((phi-eps_rad)/2)/tan(phi-eps_rad)
ax.plot([0,0],[y_offset_style,y_offset_style-l_min],c='darkblue',lw=2,marker='_',solid_capstyle='butt')
ax.text(0,y_offset_style-l_min/2,'longueur min. du style',c='darkblue',rotation=90,va='center')

# --------------------------------
# Affichage des solstices et équinoxes :
x,y=coordXY(np.radians(-25), 0)
ax.text(x,y,"solstice d'hiver",ha='center',c=coul_arcs_diurnes)
x,y=coordXY(np.radians(35), 0)
ax.text(x,y,"solstice d'été",ha='center',c=coul_arcs_diurnes)
x,y=coordXY(-0.03, 0)
ax.text(x,y,"équinoxes",ha='center',c=coul_arcs_diurnes)

#  titre :
ax.text(0,-0.85*R,'diagramme solaire stéréographique',fontsize=16,ha='center')

ax.text(0.98,0.01,'D. Alberto (www.astrolabe-science.fr)',rotation=90,fontsize=8,
        transform=ax.transAxes)
#  --------------------------------------------------
# légendes des dates :
"""
Cette partie place des points colorés sur les arcs diurnes, et 
écrit les dates correspondantes en légende.
"""
couleurs_dates = ['indigo','dodgerblue','forestgreen','cornflowerblue','darkviolet',
                  'darkslategrey','peru']
liste_dates = ['21 décembre','20 janvier - 22 novembre','19 février - 23 octobre',
               '20 mars - 23 septembre','20 avril - 23 août','21 mai - 23 juillet',
               '21 juin']
liste_decl =  np.radians([-eps,-20.15,-11.47,0,11.47,20.15,eps])
taille_pt = 15
angle_H = np.radians(0)# ligne d'angle horaire des points
x_legende=-0.5*R
y_legende=-0.4*R

for i in range(len(liste_decl)):
    X,Y = coordXY(liste_decl[i], angle_H)
    ax.scatter(X,Y,s=taille_pt,c=couleurs_dates[i])
    ax.scatter(x_legende,y_legende-0.3*i,s=taille_pt,c=couleurs_dates[i])
    ax.text(x_legende + 0.2,y_legende-0.3*i,liste_dates[i],c=couleurs_dates[i],
            fontsize=6,va='center')

#  --------------------------------------------------
#  Insertion d'image fisheye :

# logo = plt.imread('Fisheye_4.jpeg')
# imagebox = OffsetImage(logo, zoom=2.82)
# ab = AnnotationBbox(imagebox, (0, 0),zorder=0,alpha=0.5)
# ax.add_artist(ab)

#  --------------------------------------------------
#  tracé d'une grille de hauteur :

# theta=np.radians(-60) # angle des graduations de hauteur
# for h in np.arange(0,90,10):
#     h_rad=np.radians(h)
#     rayon = R * tan ((pi/2-h_rad)/2)
#     cercle = plt.Circle((0,0), radius=rayon,fill=False, color='gray',lw=0.5,alpha=0.5)
#     ax.add_patch(cercle)
#     ax.text(rayon * cos(theta),rayon * sin(theta),str(h),c='gray',
#             rotation=90+np.degrees(theta),va='center',ha='center',
#             fontsize=7,bbox=dict(boxstyle='round,pad=0.2',
#             facecolor='white',edgecolor='gray',linewidth=0.5))

#  --------------------------------------------------
fig.savefig(NomFichier + '.png',dpi=300)
fig.savefig(NomFichier + '.pdf')

